/**********************************************************************
 * $Source: /export/CVS/jameica_exampleplugin/src/de/willuhn/jameica/example/gui/control/ProjectControl.java,v $
 * $Revision: 1.8 $
 * $Date: 2006/04/03 17:09:59 $
 * $Author: web0 $
 * $Locker:  $
 * $State: Exp $
 *
 * Copyright (c) by willuhn.webdesign
 * All rights reserved
 *
 **********************************************************************/

package de.willuhn.jameica.example.gui.control;

import java.rmi.RemoteException;
import java.util.Date;

import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.willuhn.datasource.GenericIterator;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.example.Settings;
import de.willuhn.jameica.example.gui.action.TaskDetail;
import de.willuhn.jameica.example.gui.menu.ProjectListMenu;
import de.willuhn.jameica.example.gui.menu.TaskListMenu;
import de.willuhn.jameica.example.rmi.Project;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.dialogs.CalendarDialog;
import de.willuhn.jameica.gui.formatter.CurrencyFormatter;
import de.willuhn.jameica.gui.formatter.DateFormatter;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.jameica.gui.input.DecimalInput;
import de.willuhn.jameica.gui.input.DialogInput;
import de.willuhn.jameica.gui.input.Input;
import de.willuhn.jameica.gui.input.LabelInput;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.ContextMenuItem;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;


/**
 * @author willuhn
 */
public class ProjectControl extends AbstractControl
{

  // list of all projects
  private TablePart projectList;

  // Input fields for the project attributes,
  private Input name;
  private Input description;
  private Input email;
  private Input price;
  private DialogInput startDate;
  private DialogInput endDate;

  private Input effortSummary;

	// list of tasks contained in this project
	private TablePart taskList;

  // this is the currently opened project
  private Project project;

  /**
   * ct.
	 * @param view this is our view (the welcome screen).
	 */
	public ProjectControl(AbstractView view)
	{
		super(view);
	}

  /**
   * Small helper method to get the current project.
   * @return
   */
  private Project getProject()
  {
    if (project != null)
      return project;
    project = (Project) getCurrentObject();
    return project;
  }

  /**
   * Returns the input field for the project name.
   * @return input field.
   * @throws RemoteException
   */
  public Input getName() throws RemoteException
  {
    if (name != null)
      return name;
    // "255" is the maximum length for this input field.
    name = new TextInput(getProject().getName(),255);
    return name;
  }

  /**
   * Returns the input field for the project description.
   * @return input field.
   * @throws RemoteException
   */
  public Input getDescription() throws RemoteException
  {
    if (description != null)
      return description;
    description = new TextAreaInput(getProject().getDescription());
    return description;
  }

  /**
   * Returns the input field for the project email.
   * @return input field.
   * @throws RemoteException
   */
  public Input getEmail() throws RemoteException
  {
    if (email != null)
      return email;
    email = new TextInput(getProject().getEmail(),255);
    return email;
  }

  /**
   * Returns the input field for the project price.
   * @return input field.
   * @throws RemoteException
   */
  public Input getPrice() throws RemoteException
  {
    if (price != null)
      return price;
    price = new DecimalInput(getProject().getPrice(), Settings.DECIMALFORMAT);
    price.setComment(Settings.CURRENCY + " " + Settings.i18n().tr("per Hour"));
    return price;
  }

  /**
   * Returns the input field for the start date.
   * @return input field.
   * @throws RemoteException
   */
  public Input getStartDate() throws RemoteException
  {
    if (startDate != null)
      return startDate;
    
    // this is a custom dialog that show a calendar widget.
    CalendarDialog d = new CalendarDialog(CalendarDialog.POSITION_MOUSE);
    d.setTitle(Settings.i18n().tr("Choose a start date"));

    // we have to add a close listener to display the choosen
    // date in the right format.
    d.addCloseListener(new Listener()
    {
      public void handleEvent(Event event)
      {
        if (event == null || event.data == null)
          return;

        startDate.setText(Settings.DATEFORMAT.format((Date) event.data));
      }
    });

    Date initial = getProject().getStartDate();
    String s = "";
    if (initial != null)
      s = Settings.DATEFORMAT.format(initial);

    // Dialog-Input is an Input field that gets its data from a dialog.
    startDate = new DialogInput(s,d);

    //  we store the initial value
    startDate.setValue(initial);
    
    // the user shouldn't be able to enter dates by hand
    startDate.disableClientControl();
    return startDate;
  }


  /**
   * Returns the input field for the end date.
   * @return input field.
   * @throws RemoteException
   */
  public Input getEndDate() throws RemoteException
  {
    if (endDate != null)
      return endDate;
    
    CalendarDialog d = new CalendarDialog(CalendarDialog.POSITION_MOUSE);
    d.setTitle(Settings.i18n().tr("Choose an end date"));
    d.addCloseListener(new Listener()
    {
      public void handleEvent(Event event)
      {
        if (event == null || event.data == null)
          return;

        endDate.setText(Settings.DATEFORMAT.format((Date) event.data));
      }
    });

    Date ed = getProject().getEndDate();
    endDate = new DialogInput(ed == null ? "" : Settings.DATEFORMAT.format(ed),d);
    endDate.disableClientControl();
    endDate.setValue(ed);
    return endDate;
  }

	/**
	 * Returns a text label that contains the summary of all efforts in this project.
   * @return label.
   * @throws RemoteException
   */
  public Input getEffortSummary() throws RemoteException
	{
		if (effortSummary != null)
			return effortSummary;

		double sum = getProject().getEfforts() * getProject().getPrice();

		effortSummary = new LabelInput(Settings.DECIMALFORMAT.format(sum));
		effortSummary.setComment(
			Settings.CURRENCY + " [" +
			Settings.DECIMALFORMAT.format(getProject().getEfforts()) + " " +
			Settings.i18n().tr("Hours") + "]");

		return effortSummary;
	}

  /**
   * Creates a table containing all projects.
   * @return a table with projects.
   * @throws RemoteException
   */
  public Part getProjectsTable() throws RemoteException
  {
    // do we have an allready created table?
    if (projectList != null)
      return projectList;
   
    // 1) get the dataservice
    DBService service = Settings.getDBService();
    
    // 2) now we can create the project list.
    //    We do not need to specify the implementing class for
    //    the interface "Project". Jameicas Classloader knows
    //    all classes an finds the right implementation automatically. ;)
    DBIterator projects = service.createList(Project.class);
    
    // 4) create the table
    projectList = new TablePart(projects,new de.willuhn.jameica.example.gui.action.ProjectDetail());

    // 5) now we have to add some columns.
    projectList.addColumn(Settings.i18n().tr("Name of project"),"name"); // "name" is the field name from the sql table.

    // 6) the following fields are a date fields. So we add a date formatter. 
    projectList.addColumn(Settings.i18n().tr("Start date"),"startdate",new DateFormatter(Settings.DATEFORMAT));
    projectList.addColumn(Settings.i18n().tr("End date"),"enddate",    new DateFormatter(Settings.DATEFORMAT));

    // 7) calculated project price (price per hour * hours)
    projectList.addColumn(Settings.i18n().tr("Efforts"),"summary", new CurrencyFormatter(Settings.CURRENCY,Settings.DECIMALFORMAT));

		// 8) we are adding a context menu
		projectList.setContextMenu(new ProjectListMenu());
    return projectList;
  }

	/**
	 * Returns a list of tasks in this project.
   * @return list of tasks in this project
   * @throws RemoteException
   */
  public Part getTaskList() throws RemoteException
	{
		if (taskList != null)
			return taskList;

		GenericIterator tasks = getProject().getTasks();
		taskList = new TablePart(tasks,new TaskDetail());
		taskList.addColumn(Settings.i18n().tr("Task name"),"name");
		taskList.addColumn(Settings.i18n().tr("Effort"),"effort",new Formatter()
    {
      public String format(Object o)
      {
      	if (o == null)
      		return "-";
        return o + " h";
      }
    });
    
		TaskListMenu tlm = new TaskListMenu();

		// we add an additional menu item to create tasks with predefined project.
		tlm.addItem(new ContextMenuItem(Settings.i18n().tr("Create new task within this Project"),new Action()
    {
      public void handleAction(Object context) throws ApplicationException
      {
      	new TaskDetail().handleAction(getProject());
      }
    }));
    taskList.setContextMenu(tlm);
    taskList.setSummary(false);
    return taskList;
	}

  /**
   * This method stores the project using the current values. 
   */
  public void handleStore()
  {
    try
    {

      // get the current project.
      Project p = getProject();

      // invoke all Setters of this project and assign the current values
      p.setDescription((String) getDescription().getValue());

			// we can cast the return value of the calendar fields directly to "java.util.Date".
      p.setEndDate((Date) getEndDate().getValue());
      p.setStartDate((Date) getStartDate().getValue());

      p.setName((String) getName().getValue());

			// the DecimalInput fields returns a Double object
      Double d = (Double) getPrice().getValue();
      p.setPrice(d == null ? 0.0 : d.doubleValue());

      // Now, let's store the project
      // The store() method throws ApplicationExceptions if
      // insertCheck() or updateCheck() failed.
      try
      {
        p.store();
        GUI.getStatusBar().setSuccessText(Settings.i18n().tr("Project stored successfully"));
      }
      catch (ApplicationException e)
      {
        GUI.getView().setErrorText(e.getMessage());
      }
    }
    catch (RemoteException e)
    {
      Logger.error("error while storing project",e);
      GUI.getStatusBar().setErrorText(Settings.i18n().tr("Error while storing Project"));
    }
  }
}


/**********************************************************************
 * $Log: ProjectControl.java,v $
 * Revision 1.8  2006/04/03 17:09:59  web0
 * *** empty log message ***
 *
 * Revision 1.7  2005/05/07 15:27:39  web0
 * *** empty log message ***
 *
 * Revision 1.6  2004/11/12 18:03:46  willuhn
 * *** empty log message ***
 *
 * Revision 1.5  2004/11/05 17:34:23  willuhn
 * *** empty log message ***
 *
 * Revision 1.4  2004/11/05 01:54:51  willuhn
 * *** empty log message ***
 *
 * Revision 1.3  2004/11/05 00:28:26  willuhn
 * *** empty log message ***
 *
 * Revision 1.2  2004/11/04 23:58:57  willuhn
 * *** empty log message ***
 *
 * Revision 1.1  2004/11/04 19:30:04  willuhn
 * *** empty log message ***
 *
 * Revision 1.1  2004/11/03 20:05:40  willuhn
 * *** empty log message ***
 *
 * Revision 1.2  2004/06/30 20:57:52  willuhn
 * *** empty log message ***
 *
 * Revision 1.1  2004/06/11 00:07:15  willuhn
 * @N Example classes
 *
 **********************************************************************/