/**********************************************************************
 * $Source: /export/CVS/jameica_exampleplugin/src/de/willuhn/jameica/example/gui/control/TaskControl.java,v $
 * $Revision: 1.2 $
 * $Date: 2004/11/12 18:03:46 $
 * $Author: willuhn $
 * $Locker:  $
 * $State: Exp $
 *
 * Copyright (c) by willuhn.webdesign
 * All rights reserved
 *
 **********************************************************************/
package de.willuhn.jameica.example.gui.control;

import java.rmi.RemoteException;

import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.willuhn.datasource.GenericIterator;
import de.willuhn.jameica.example.Settings;
import de.willuhn.jameica.example.rmi.Project;
import de.willuhn.jameica.example.rmi.Task;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.dialogs.ListDialog;
import de.willuhn.jameica.gui.formatter.DateFormatter;
import de.willuhn.jameica.gui.input.DecimalInput;
import de.willuhn.jameica.gui.input.DialogInput;
import de.willuhn.jameica.gui.input.Input;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Controller for the task view.
 */
public class TaskControl extends AbstractControl
{

	// the current task object
	private Task task;

	// the input fields for the task.
	private DialogInput project;
	private Input name;
	private Input effort;
	private Input comment;

  /**
   * ct.
   * @param view
   */
  public TaskControl(AbstractView view)
  {
    super(view);
  }

	/**
	 * Returns the current task.
   * @return the task.
   */
  private Task getTask()
	{
		if (task != null)
			return task;
		task = (Task) getCurrentObject();
		return task;
	}

	/**
	 * Returns a the field to choose the project.
   * @return the project.
   * @throws RemoteException
   */
  public Input getProject() throws RemoteException
	{
		if (project != null)
			return project;
		
		// this custom widget contains two parts.
		// 1) a readonly text field to show up the choosen project name
		// 2) a button behind that opens a dialog, containing a list of existing projects.
		
		// first we have to create the dialog
		GenericIterator projects = Settings.getDBService().createList(Project.class);
		ListDialog d = new ListDialog(projects,ListDialog.POSITION_MOUSE);
		d.setTitle(Settings.i18n().tr("Choose a project"));
		d.addColumn(Settings.i18n().tr("Project name"),"name");
		d.addColumn(Settings.i18n().tr("Start date"),"startdate", new DateFormatter(Settings.DATEFORMAT));
		d.addColumn(Settings.i18n().tr("End date"),"enddate", new DateFormatter(Settings.DATEFORMAT));
		d.addCloseListener(new Listener()
    {
			// The text field doesn't know, what to display after choosing the project
			// So add a close listener to the dialog, catch the project and set it's
			// name by hand into the text field.
      public void handleEvent(Event event)
      {
      	if (event == null || event.data == null)
      		return;
      	if (!(event.data instanceof Project))
      		return;
      		
				try
				{
					project.setText(((Project)event.data).getName());
				}
				catch (RemoteException e)
				{
					Logger.error("erro while choosing project",e);
					GUI.getStatusBar().setErrorText(Settings.i18n().tr("Error while choosing project"));
				}
      }
    });
    
    Project p = getTask().getProject();

		// the initial choosen project
    String s = p == null ? "" : p.getName();
    project = new DialogInput(s,d);
    project.setValue(p); // we store the initial value

		// we disable the client control (the text field)
		// the user shouldn't be able to enter a project name by hand
    project.disableClientControl();
    return project;
	}

	/**
	 * Returns an input field for the task name.
   * @return input field.
   * @throws RemoteException
   */
  public Input getName() throws RemoteException
	{
		if (name != null)
			return name;
		// "255" is the maximum length of the name attribute.
		name = new TextInput(getTask().getName(),255);
		return name;
	}

	/**
	 * Returns an input field for the task effort.
   * @return input field.
   * @throws RemoteException
   */
  public Input getEffort() throws RemoteException
	{
		if (effort != null)
			return effort;

		// we assign our system decimal formatter
		effort = new DecimalInput(getTask().getEffort(),Settings.DECIMALFORMAT);
		effort.setComment(Settings.i18n().tr("Hours [example: enter \"0,5\" to store 30 minutes]"));
		return effort;
	}

	/**
	 * Returns an input field for the task comment.
   * @return input field.
   * @throws RemoteException
   */
  public Input getComment() throws RemoteException
	{
		if (comment != null)
			return comment;
		comment = new TextAreaInput(getTask().getComment());
		return comment;
	}

	/**
	 * This method stores the task using the current values. 
	 */
	public void handleStore()
	{
		try
		{

			// get the current task.
			Task t = getTask();

			// invoke all Setters of this task and assign the current values
			t.setName((String) getName().getValue());

			// we can cast the value of the project dialogInput directly to "Project".
			t.setProject((Project) getProject().getValue());

			// the DecimalInput fields returns a Double object
			Double d = (Double) getEffort().getValue(); 
			t.setEffort(d == null ? 0.0 : d.doubleValue());

			t.setComment((String) getComment().getValue());

			// Now, let's store the project
			// The store() method throws ApplicationExceptions if
			// insertCheck() or updateCheck() failed.
			try
			{
				t.store();
				GUI.getStatusBar().setSuccessText(Settings.i18n().tr("Task stored successfully"));
			}
			catch (ApplicationException e)
			{
				GUI.getView().setErrorText(e.getMessage());
			}
		}
		catch (RemoteException e)
		{
			Logger.error("error while storing task",e);
			GUI.getStatusBar().setErrorText(Settings.i18n().tr("Error while storing Task"));
		}
	}

}


/**********************************************************************
 * $Log: TaskControl.java,v $
 * Revision 1.2  2004/11/12 18:03:46  willuhn
 * *** empty log message ***
 *
 * Revision 1.1  2004/11/04 23:58:57  willuhn
 * *** empty log message ***
 *
 **********************************************************************/